--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG, Lily, Lyn and Roth.
-- by rabbit.magtheridon@gmail.com
--
--]]

-- SETUP --
local maincolor = {80/255, 64/255, 77/255}		-- mainly used for healthbar
--local sndcolor = {208/255, 172/255, 146/255}	-- font color, ...
local trdcolor = {220/255, 220/255, 220/255}	-- castbar, buff/debuff borders

local mediaFolder = "Interface\\AddOns\\oUF_Rabbit\\media\\"
local Auratex = mediaFolder.."dBBorderH"

local glowtex = mediaFolder .. "glowTex"
local NumbFont = mediaFolder.."Prototype.ttf"
local NumbFS = 12 
local FontF = "THINOUTLINE" --local fontF = nil

local statusbarTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\inset"
local borderTexture = "Interface\\AddOns\\oUF_Rabbit\\media\\border2"
local font = "Interface\\AddOns\\oUF_Rabbit\\media\\font.ttf"
local upperfont = "Interface\\AddOns\\oUF_Rabbit\\media\\upperfont.ttf"
local _, pClass = UnitClass("player")

-- The width of the player, target and pet frames.
local frameWidth = 230

local LSM = LibStub("LibSharedMedia-3.0", true)
if LSM then
	LSM:Register("statusbar", "oUF_Rabbit", statusbarTexture)
end

local function menu(self)
	local unit = self.unit:sub(1, -2)
	local cunit = self.unit:gsub("^%l", string.upper)
	if cunit == "Vehicle" then cunit = "Pet" end
	if unit == "party" or unit == "partypet" then
		ToggleDropDownMenu(1, nil, _G["PartyMemberFrame"..self.id.."DropDown"], "cursor", 0, 0)
	elseif _G[cunit.."FrameDropDown"] then
		ToggleDropDownMenu(1, nil, _G[cunit.."FrameDropDown"], "cursor", 0, 0)
	end
end

local grey = {0.5, 0.5, 0.5}
local white = {1, 1, 1}
local name = "%s %s"
local classifications = {
	worldboss = "??",
	rareelite = "%s*+",
	rare = "%s*",
	elite = "%s+",
}
local function updateName(self, unit)
	if not self.Name then return end
	if self.unit == unit then
		if unit == "target" then
			local cl = UnitClassification(unit)
			local level = classifications[cl] and classifications[cl]:format(UnitLevel(unit)) or UnitLevel(unit)
			local n = UnitName(unit)
			self.Name:SetText(name:format(level, n))
		else
			local n = UnitName(unit)
			if unit == "targettarget" or unit == "focus" then
				self.Name:SetText(n:sub(1, 3))
			else
				self.Name:SetText(n)
			end
		end

		local c = (not UnitIsConnected(unit) or UnitIsGhost(unit) or UnitIsDead(unit)) and grey or white
		self.Name:SetTextColor(unpack(c))
	end
end

local function short(v)
	if v <= 9999 then return v end
	if v >= 1000000 then
		local value = string.format("%.1fm", v/1000000)
		return value
	elseif v >= 10000 then
		local value = string.format("%.1fk", v/1000)
		return value
	end
end

local targetFormat = "%s/%s"
local function updateHealth(bar, unit, min, max)
	if UnitIsDead(unit) then
		bar.value:SetText("Dead")
		bar.value:SetTextColor(.5, .5, .5)
	elseif UnitIsGhost(unit) then
		bar.value:SetText("Ghost")
		bar.value:SetTextColor(.5, .5, .5)
	elseif not UnitIsConnected(unit) then
		bar.value:SetText("Off")
		bar.value:SetTextColor(.5, .5, .5)
	else
		local value = nil
		if unit == "focus" or unit == "targettarget" then
			value = math.floor(min/max*100+0.5) .. "%"
		elseif unit == "target" then
			value = targetFormat:format(short(min), short(max))
		else
			if max == min then
				value = short(max)
			else
				value = "-" .. short(max - min)
			end
		end
		if bar.percent then
			bar.percent:SetText(math.floor(min / max * 100 + 0.5) .. "%")
		end
		bar.value:SetText(value)
		bar.value:SetTextColor(1, 1, 1)
		bar:SetStatusBarColor(101 / 255, 146 / 255, 71 / 255, 1)
	end
	updateName(bar:GetParent(), unit)
end

local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground", tile = true, tileSize = 16,
	insets = {left = -4, right = -4, top = -4, bottom = -4},
}

local function onEnter(...)
	if InCombatLockdown() then return end
	UnitFrame_OnEnter(...)
end

local function onLeave(...)
	if InCombatLockdown() then return end
	UnitFrame_OnLeave(...)
end

local function power(self)
	local pp = CreateFrame("StatusBar")
	pp:SetHeight(8)
	self.Health:SetHeight(self:GetHeight() - 12)
	pp:SetStatusBarTexture(statusbarTexture)
	pp:SetStatusBarColor(.25, .25, .35)

	pp:SetParent(self)
	pp:SetPoint("BOTTOM", self)
	pp:SetPoint("LEFT")
	pp:SetPoint("RIGHT")

	pp.colorTapping = true
	pp.colorDisconnected = true
	pp.colorClass = true
	pp.colorClassNPC = true
	pp.colorClassPet = true

	local ppbg = pp:CreateTexture(nil, "BORDER")
	ppbg:SetAllPoints(pp)
	ppbg:SetAlpha(.3)
	ppbg:SetTexture(statusbarTexture)
	pp.bg = ppbg

	self.Power = pp
end

local function namestring(self)
	local name = self.Health:CreateFontString(nil, "OVERLAY")
	name:SetPoint("LEFT", 4, 0)
	name:SetPoint("RIGHT", self.Health.value, "LEFT")
	name:SetJustifyH("LEFT")
	name:SetFont(font, 14, "OUTLINE")
	name:SetShadowOffset(1, -1)
	self.Name = name
end

local function auras(self)
	local w = self:GetWidth()
	self.Auras = CreateFrame("Frame", nil, self)
	self.Auras.showDebuffType = false
	self.Auras.disableCooldown = true
	self.Auras.size = w/9-4
	self.Auras:SetHeight((w/10)*4+6)
	self.Auras:SetWidth(w)
	self.Auras.spacing = 0
	self.Auras.gap = true
	self.Auras:SetPoint("BOTTOMLEFT", self, "TOPLEFT", 0, 26)
	self.Auras.initialAnchor = "BOTTOMLEFT"
	self.Auras["growth-x"] = "RIGHT"
	self.Auras["growth-y"] = "UP"
end

local cbBackdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	insets = {top = -3, left = -3, bottom = -3, right = -3}
}
local function castbar(self, unit)
	local cb = CreateFrame("StatusBar", nil, self)
	cb:SetStatusBarTexture(statusbarTexture)
	cb:SetBackdrop(cbBackdrop)
	if unit == "player" then
		local color = RAID_CLASS_COLORS[pClass]
		cb:SetStatusBarColor(color.r - 0.2, color.g - 0.2, color.b - 0.2)
		cb:SetHeight(12)
		cb.SafeZone = cb:CreateTexture(nil,"ARTWORK")
		cb.SafeZone:SetTexture(statusbarTexture)
		cb.SafeZone:SetVertexColor(.75, .10, .10, .6)
		cb.SafeZone:SetPoint("TOPRIGHT")
		cb.SafeZone:SetPoint("BOTTOMRIGHT")

		cb:SetPoint("BOTTOMLEFT", self, "TOPLEFT", -2, 12)
		cb:SetPoint("BOTTOMRIGHT", self, "TOPRIGHT", 2, 12)
	elseif unit == "target" then
		cb:SetStatusBarColor(0.1, 0.8, 0.3)
		cb:SetWidth(240)
		cb:SetHeight(24)
		cb:SetPoint("CENTER", UIParent, 0, -100)
	elseif unit == "focus" then
		cb:SetStatusBarColor(0.80, 0.01, 0)
		cb:SetPoint("CENTER", UIParent, 0, -60)
		cb:SetWidth(500)
		cb:SetHeight(24)
	end
	if unit == "target" or unit == "focus" then
		cb.Text = cb:CreateFontString(nil, "OVERLAY")
		cb.Text:SetAllPoints(cb)
		cb.Text:SetFont(font, 16, "OUTLINE")
		cb.Text:SetShadowOffset(1, -1)
		cb.Text:SetTextColor(1, 1, 1)
		cb.Text:SetJustifyH("CENTER")
	end

	cb:SetBackdropColor(0, 0, 0, 0.5)

	cb.bg = cb:CreateTexture(nil, "BORDER")
	cb.bg:SetAllPoints(cb)
	cb.bg:SetTexture(0, 0, 0, 0.6)

	cb.Spark = cb:CreateTexture(nil, "OVERLAY")
	cb.Spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
	cb.Spark:SetVertexColor(1, 1, 1, 1)
	cb.Spark:SetBlendMode("ADD")
	cb.Spark:SetWidth(30)
	cb.Spark:SetHeight(cb:GetHeight() * 2)
	cb.Spark:Show()

	CreateBorder(cb, 12, 4)

	self.Castbar = cb
end

local function iconHide(self) self:SetVertexColor(trdcolor[1], trdcolor[2], trdcolor[3]) end
local bd = {edgeFile = glowtex, edgeSize = 5, insets = {left = 4, right = 4, top = 4, bottom = 4}}
local function auraIcon(icons, icon)
	icon.count:SetFont(NumbFont, NumbFS*0.8, FontF)
	icon.count:SetPoint('TOP', icon, 'BOTTOM', 0, 10)
	icon.count:SetJustifyH('CENTER')

	icon.icon:SetTexCoord(.07, .93, .07, .93)
	icon.icon:SetPoint("TOPLEFT", icon, 2, -2)
	icon.icon:SetPoint("BOTTOMRIGHT", icon, -2, 2)
	
	icon.Glow = CreateFrame("Frame", nil, icon)
	icon.Glow:SetPoint("TOPLEFT", icon, -4, 4)
	icon.Glow:SetPoint("BOTTOMRIGHT", icon, 4, -4)
	icon.Glow:SetFrameStrata("BACKGROUND")
	icon.Glow:SetBackdrop(bd)
	icon.Glow:SetBackdropColor(0, 0, 0, 0)
	icon.Glow:SetBackdropBorderColor(0, 0, 0)

	icon.overlay:SetTexture(Auratex)
	icon.overlay:SetTexCoord(0,1,0,1)
	icon.overlay.Hide = iconHide
end

local alwaysShow = {
	[GetSpellInfo(7386)] = true, -- Sunder Armor
	[GetSpellInfo(64382)] = true, -- Shattering Throw
}
local playerUnits = {
	player = true,
	pet = true,
	vehicle = true,
}

local function auraUpdateIcon(icons, unit, icon, index, offset)
	if icon.debuff and unit == "target" then
		local name, _, _, _, dtype = UnitAura(unit, index, icon.filter)
		if alwaysShow[name] or playerUnits[icon.owner] then
			local color = DebuffTypeColor[dtype] or DebuffTypeColor.none
			icon:SetBackdropColor(color.r * 0.9, color.g * 0.9, color.b * 0.9)
			icon.overlay:SetVertexColor(color.r * 0.9, color.g * 0.9, color.b * 0.9)
			icon.icon:SetDesaturated(false)
		else
			icon:SetBackdropColor(0, 0, 0)
			icon.overlay:SetVertexColor(maincolor[1], maincolor[2], maincolor[3])
			icon.icon:SetDesaturated(true)
		end
	end
end

local unitSpecific = {
	party = function(self)
		self:SetWidth(168)
		self:SetHeight(36)
		self.Health.value:SetPoint("RIGHT", -4, 0)
		namestring(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "TOP")
		power(self)
	end,
	targettarget = function(self)
		self:SetWidth((frameWidth / 2) - 8)
		self:SetHeight(20)
		self.Health.value:SetPoint("RIGHT", -4, 0)
		namestring(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "TOP")
	end,
	focus = function(self)
		self:SetWidth((frameWidth / 2) - 8)
		self:SetHeight(20)
		self.Health.value:SetPoint("RIGHT", -4, 0)
		namestring(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "TOP")
		castbar(self, "focus")
	end,
	player = function(self)
		self:SetWidth(frameWidth)
		self:SetHeight(40)

		self.Health.frequentUpdates = true
		self.Health.value:SetPoint("CENTER")
		
		self.RaidIcon:SetPoint("RIGHT", self.Health, -8, 0)
		power(self)
		castbar(self, "player")
	end,
	target = function(self)
		self:SetWidth(frameWidth)
		self:SetHeight(40)

		local hp = self.Health
		hp.frequentUpdates = true
		hp.value:SetPoint("CENTER")

		local hpx = hp:CreateFontString(nil, "OVERLAY")
		hpx:SetPoint("RIGHT", -8, 0)
		hpx:SetFont(upperfont, 14, "OUTLINE")
		hpx:SetShadowOffset(1, -1)
		hpx:SetTextColor(1, 1, 1)
		hp.percent = hpx

		local top = hp:CreateFontString(nil, "OVERLAY")
		top:SetPoint("BOTTOMLEFT", hp, "TOPLEFT", 0, 8)
		top:SetWidth(frameWidth)
		top:SetJustifyH("LEFT")
		top:SetFont(font, 16, "OUTLINE")
		top:SetShadowOffset(1, -1)
		top:SetNonSpaceWrap(true)
		self:Tag(top, "[raidcolor][name]|r")
		
		self.RaidIcon:SetPoint("LEFT", hp, 8, 0)
		
		power(self)
		
		auras(self)
		self.Auras.PostCreateIcon = auraIcon
		self.Auras.PostUpdateIcon = auraUpdateIcon
		
		castbar(self, "target")
	end,
	pet = function(self)
		self:SetWidth(frameWidth)
		self:SetHeight(20)
		if pClass == "HUNTER" then
			self.Health.colorHappiness = true
		end
		namestring(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "TOP")
	end,
	boss = function(self)
		self:SetHeight(20)
		namestring(self)
		self.RaidIcon:SetPoint("CENTER", self.Health, "TOP")
	end,
}

local function rabbitStyle(self, unit)
	self.menu = menu
	self:EnableMouse(true)
	self:SetScript("OnEnter", onEnter)
	self:SetScript("OnLeave", onLeave)
	self:RegisterForClicks("anyup")
	self:SetAttribute("*type2", "menu")

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, 0.8)

	local hp = CreateFrame("StatusBar")
	hp:SetHeight(20)
	hp:SetStatusBarTexture(statusbarTexture)
	hp:SetStatusBarColor(0, 1, 0)
	hp:SetParent(self)
	hp:SetPoint("TOP")
	hp:SetPoint("LEFT")
	hp:SetPoint("RIGHT")
	local hpbg = hp:CreateTexture(nil, "BORDER")
	hpbg:SetAllPoints(hp)
	hpbg:SetTexture(statusbarTexture)
	hpbg:SetVertexColor(.1,.4,.1,0.3)
	local hpp = hp:CreateFontString(nil, "OVERLAY")
	hpp:SetFont(upperfont, 14, "OUTLINE")
	hpp:SetShadowOffset(1, -1)
	hpp:SetTextColor(1, 1, 1)
	hp.value = hpp
	hp.colorDisconnected = true
	hp.PostUpdate = updateHealth
	self.Health = hp

	local ricon = hp:CreateTexture(nil, "OVERLAY")
	ricon:SetHeight(24)
	ricon:SetWidth(24)
	ricon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = ricon

	local u = unit:find("^boss") and "boss" or unit
	if unitSpecific[u] then
		unitSpecific[u](self)
	end

	CreateBorder(self, 12, 6)
end

oUF:RegisterStyle("Rabbit", rabbitStyle)
oUF:SetActiveStyle("Rabbit")

local player = oUF:Spawn("player")
local pet = oUF:Spawn("pet")
local target = oUF:Spawn("target")
local tot = oUF:Spawn("targettarget")
local focus = oUF:Spawn("focus")
local party = oUF:SpawnHeader("oUF_Party", nil, "custom [group:raid]hide;[group:party]show;hide",
	"showParty", true,
	"yOffset", -16
)

player:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOM", -18, 140)
target:SetPoint("BOTTOMLEFT", UIParent, "BOTTOM", 18, 140)
pet:SetPoint("TOPLEFT", player, "BOTTOMLEFT", 0, -16)
tot:SetPoint("TOPRIGHT", target, "BOTTOMRIGHT", 0, -16)
party:SetPoint("TOPLEFT", 30, -30)

local r = _G["RabbitPanelRIGHT"]
focus:SetPoint("BOTTOMLEFT", r, "TOPLEFT", 5, 12)
focus:SetPoint("BOTTOMRIGHT", r, "TOPRIGHT", -5, 12)

local boss = {}
for i = 1, MAX_BOSS_FRAMES do
	local unit = oUF:Spawn("boss"..i)
	if i == 1 then
		unit:SetPoint("BOTTOMLEFT", focus, "TOPLEFT", 0, 16)
		unit:SetPoint("BOTTOMRIGHT", focus, "TOPRIGHT", 0, 16)
	else
		unit:SetPoint("BOTTOMLEFT", boss[i-1], "TOPLEFT", 0, 16)
		unit:SetPoint("BOTTOMRIGHT", boss[i-1], "TOPRIGHT", 0, 16)
	end
	boss[i] = unit
end


