--[[
--
-- oUF_Rabbit
--
-- Custom layout for oUF.
-- Based on VuBce, rUnits, TsoHG and Lily.
-- by rabbit.magtheridon@gmail.com
--
--]]

--------------------------------------------------------------------------------
-- OPTIONS
-- I added these so that I could easily tweak it myself to try and find the best
-- combination for myself, and the options take up so little space that I just
-- left them in.

-- Where to spawn the first raid group.
local DISTANCE_FROM_LEFT = 32
local DISTANCE_FROM_TOP = -30

-- How many groups to spawn, 0-8.
local MAX_GROUPS_TO_SPAWN = 8
-- The space below and to the side of each unit box.
local UNIT_SPACING = 10

-- [percent]: When a unit drops below this set percent, the name in the center
--            will be replaced with a HP indicator. Set it to nil to disable.
local HEALTH_DISPLAY = 90
local HEALTH_DISPLAY_AS = "percent" -- "percent" or "deficit"

-- 0.1-1: Sets the scale of the raid frames.
local SCALE = 1

-- Statusbar texture used for HP and mana bars.
local statusbarTextureVertical = "Interface\\AddOns\\oUF_Rabbit\\media\\inset"

--------------------------------------------------------------------------------
-- Layout

local backdrop = {
	bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", 
	insets = { left = -2, right = -2, top = -2, bottom = -2 },
}

local percent = "%d%%"
local deficit = "-%d"
local dead = "DEAD"
local ghost = "GHOST"
local offline = "OFFLINE"

local function update(bar, unit, min, max)
	local textFrame = bar:GetParent().Name

	if UnitIsDead(unit) then
		textFrame:SetText(dead)
	elseif UnitIsGhost(unit) then
		textFrame:SetText(ghost)
	elseif not UnitIsConnected(unit) then
		textFrame:SetText(offline)
	else
		local p = floor(min / max * 100)
		if HEALTH_DISPLAY and p <= HEALTH_DISPLAY then
			if HEALTH_DISPLAY_AS == "percent" then
				textFrame:SetFormattedText(percent, p)
			else
				textFrame:SetFormattedText(deficit, max - min)
			end
		else
			textFrame:SetText(UnitName(unit))
		end
	end
	local _, c = UnitClass(unit)
	local color = RAID_CLASS_COLORS[c]
	if color then
		textFrame:SetTextColor(color.r, color.g, color.b)
	end
end

local function onEnter(...)
	if InCombatLockdown() then return end
	UnitFrame_OnEnter(...)
end

local function onLeave(...)
	if InCombatLockdown() then return end
	UnitFrame_OnLeave(...)
end

local range = {
	insideAlpha = 1,
	outsideAlpha = .4,
}

local function raid(self, unit)
	self:EnableMouse(true)
	self:SetScript("OnEnter", onEnter)
	self:SetScript("OnLeave", onLeave)
	self:RegisterForClicks("anyup")

	self:SetWidth(62.4)
	self:SetHeight(22)

	self:SetBackdrop(backdrop)
	self:SetBackdropColor(0, 0, 0, .9)

	local hp = CreateFrame("StatusBar", nil, self)
	hp:SetFrameStrata("BACKGROUND")
	hp:SetStatusBarTexture(statusbarTextureVertical, "BORDER")
	hp:SetStatusBarColor(0.8, 0.8, 0.8)
	hp:SetAllPoints(self)
	hp.PostUpdate = update
	self.Health = hp

	local name = self:CreateFontString(nil, "OVERLAY", "GameFontHighlightSmall")
	name:SetAllPoints(self)
	name:SetJustifyH("CENTER")
	name:SetShadowColor(0, 0, 0, 0.9)
	name:SetShadowOffset(1, -1)
	self.Name = name

	local icon = self:CreateTexture(nil, "OVERLAY")
	icon:SetHeight(16)
	icon:SetWidth(16)
	icon:SetPoint("TOP", self, 0, 10)
	icon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
	self.RaidIcon = icon

	self.Range = range

	CreateBorder(self, 10, 4)

	return self
end

oUF:RegisterStyle("Rabbit_Raid", raid)
oUF:SetActiveStyle("Rabbit_Raid")

local raid = {}
for i = 1, 5 do
	local rGroup = oUF:SpawnHeader(nil, nil, "raid")
	rGroup:SetAttribute("groupFilter", tostring(i))
	rGroup:SetAttribute("showRaid", true)
	rGroup:SetAttribute("point", "BOTTOM")
	rGroup:SetAttribute("yOffset", UNIT_SPACING)
	table.insert(raid, rGroup)
	if i == 1 then
		rGroup:SetPoint("BOTTOMLEFT", UIParent, 30, 200)
	else
		rGroup:SetPoint("BOTTOMLEFT", raid[i - 1], "BOTTOMRIGHT", UNIT_SPACING, 0)
	end
	rGroup:SetScale(SCALE)
	rGroup:Show()
end

